/*
 * GREAT-ER
 *
 * PARAMS.H
 *
 * definition of parameter structures
 *
 * author =  Geert Boeije
 * state =   07.04.1999
*/

#ifndef PARAMS_H
#define PARAMS_H


/*
 * ************************************************
 * FIXED constants
 * ************************************************
*/

/*
 * -----------------------
 * fixed by law of nature 
 * -----------------------
*/

#define GAS_CONSTANT                8.314
#define KELVIN                      273.15

/*
 * -----------------------
 * fixed as default
 * -----------------------
*/

#define DEFAULT_LAKE_DEPTH          10        /* lake is considered 10m deep 
                                                 if no depth is given */


/*
 * ************************************************
 * NON GEO REFERENCED 
 *
 * GENERAL PARAMETERS
 * ************************************************
*/

typedef struct
{ 
  FloatType

  /* how to deal with groundwater leaks */

  groundwater_conservative,      /* 1 or 0    leaks before or after submodels */


  /* general environmental information */
  /*-----------------------------------*/

  temp_water,     /* degr C */
  temp_sewage,    /* degr C */
  temp_air,       /* degr C */
  wind_speed,     /* m/s         speed of wind above river */
  h_air,          /* m           air mixing height */

  /* sewage properties */

  SS_sewage,      /* mg/L        suspended solids in sewage */
  f_oc_sewage,    /* -           sewage solids OC fraction */
  ro_SS_sewage,   /* kg/L        sewage solids density */

  PHI_cap_BOD,    /* g/cap.d     per capita BOD flux */
  PHI_cap_N,      /* g/cap.d     per capita N flux */

  /* primary settler properties */

  R_prim_BOD,     /* -           BOD removal efficiency in primary settler */
  R_prim_N,       /* -           N removal efficiency in primary settler */

  /* activated sludge properties */

  r_end,          /* g/g.h       act. sludge endogenous respiration rate */
  f_oc_ML,        /* -           ML solids OC fraction */
  ro_SS_ML,       /* kg/L        ML solids density */

  PSI,            /* -           empirical constant for surface aeration */
  kGa_kLa,        /* -           kGa / kLa */

  ML_salinity,    /* mgCl/L      salinity of mixed liquor */

  /* combined sewer flow estimation */
  /*--------------------------------*/

  CS_Q_DWF,       /* -           combined sewer flow as fraction of dry weather flow */


  /* several defaults */
  /*------------------*/

  /* mass transfer */

  K_air,          /* m/s         mass transfer in air */
  K_water,        /* m/s         mass transfer in water */

  /* (de)sorption rates */
  
  k_sorb_prim,    /* 1/s         1st order (de)sorprion rate - prim settler */
  k_sorb_ML,      /* 1/s         1st order (de)sorprion rate - ML */
  k_sorb_sec,     /* 1/s         1st order (de)sorprion rate - sec settler */

  /* settler efficiency - empirical constants */

  a_prim,         /* -           primary: effect of Q/DWF on solids removal */
  a_sec,          /* -           secondary: effect of Q/DWF on solids removal */


  /* correlations */
  /*--------------*/

  temp_corr,      /*          correlation of temp to flow (both air and water) */
  wind_corr,      /*          correlation of wind speed to flow */
  SS_corr,        /*          correlation of SS to flow */
  DO_corr,        /*          correlation of DO to flow */
  BOD_corr,       /*          correlation of BOD to flow */
  k_SS_corr;      /*          correlation of k_SS to flow */
}
*NonGeoRef_ParPtr;

#define INDEX_TEMP_WATER            1
#define INDEX_TEMP_SEWAGE           2
#define INDEX_TEMP_AIR              3
#define INDEX_WINDSPEED             4
#define INDEX_CS_Q_DWF              19

#define NR_NON_GEOREF_PARAMS        33     /* nr. of non-geo-referenced pars.
                                              in input file */



/*
 * ************************************************
 * CHEMICAL  PARAMETERS
 * ************************************************
*/

typedef struct
{ 
  FloatType
  
  /* ------------------- */
  /* chemical properties */
  /* ------------------- */

  /* physical / chemical */

  MM,             /* g/mole   molar mass */
  K_ow,           /* -        octanol / water partitioning */
  P_vapour,       /* Pa       vapour pressure */
  solubility,     /* mg/L     water solubility */
  pKa,            /* -        dissociation constant */
  a,              /* 1 or -1  acid vs. basic disociation */

  /* override default estimations of Kd and H */

  Kd_river,       /* L/kg     solids / liquid partitioning in river */  
  Kd_sewage,      /* L/kg     solids / liquid partitioning in sewage */  
  Kd_ML,          /* L/kg     solids / liquid partitioning in ML */  

  H,              /* Pa.m3/mol    Henry's coefficient */

  /* biodegradation */

  k_std_biodeg,   /* (mg/L)-1.h-1  
                              standard 2ble first order 
                              aerobic biodegradation rate */
  a_anaerobic,    /* -        anaerobic biodegradation correction factor */
  a_anoxic,       /* -        anoxic biodegradation correction factor */
  K_DO,           /* mg/L     monod constant for aerobic / anaerobic */
  a_sorbed,       /* -        sorbed biodegradatin correction factor */
  Q10,            /* -        temperature correction for biodegradation */

    /* this is Monod stuff for WWTP model */

  use_Monod,      /* 1 or 0   Monod - use it (1) or use pseudo first order (0) */
  k_max,          /* h-1      Monod - max. biodegradation rate */
  Y,              /* -        Monod - biomass Yield */
  K_s,            /* mg/L     Monod - affinity constant */
  b,              /* h-1      Monod - biomass decay rate */



  /* ----------- */
  /* RIVER MODEL */
  /* ----------- */

  /* mode 1 */

  k_instream,     /* h-1      RIVER - default in stream removal rate */

  /* mode 2 */

  k_vol,          /* h-1      RIVER - default volatilisation rate */
  k_deg,          /* h-1      RIVER - default degradation rate */

  /* mode 3 */

  m3_vol,         /* 1/0      volatilization on/off */
  m3_sed,         /* 1/0      sedimentation on/off */
  m3_deg_bio,     /* 1/0      biodegradation on/off */
  m3_deg_photo,   /* 1/0      photodegradation on/off */
  m3_deg_hydr,    /* 1/0      hydrolysis on/off */

  k_hydr_n,       /* h-1      hydrolysis rate - neutral */
  k_hydr_b,       /* h-1      hydrolysis rate - base */
  k_hydr_a,       /* h-1      hydrolysis rate - acid */
  k_photo,        /* h-1      photolysis rate - near surface */

  a_river,        /* -        correction factor for biodegradation in rivers */


  /* ---------- */
  /* WWTP MODEL */
  /* ---------- */

  /* mode 1 !*/

  R_primary,            /* -        removal in primary settler */
  R_activated_sludge,   /* -        removal in act. sludge */
  R_trickling_filter,   /* -        removal in TF */

  /* mode 2, mode 3 */

  a_AS,                 /* -        correction for biodegradation in act. sludge */
  k_WWTP_nonbio,        /* h-1      rate for nonbiological degradation in WWTPs */


  /* ----------- */
  /* SEWER MODEL */
  /* ----------- */

  R_sewer;              /* -        removal in sewer */

}
*Chemical_ParPtr;

#define NR_CHEMICAL_PARAMS        40     /* nr. of chemical pars. in input file */



/* chemical properties, derived once per shot */

typedef struct
{ 
  FloatType

  Kd_sewage,      /* L/kg     solids / liquid partitioning in sewage */  
  Kd_ML,          /* L/kg     solids / liquid partitioning in ML */  
  H,              /* Pa.m3/mol    Henry's coefficient */
  K_H;            /* -            dimensionless air/water xch coeff */
}
ChemProp;



/*
 * ************************************************
 * RIVER CLASS 
 *
 * PARAMETERS
 * ************************************************
*/

typedef struct
{ 
  FloatType

  RivClass_ID,          /* identification of river class */

  mode,                 /* preferred model complexity mode */

  lake,                 /* is it a lake (1/0) ? */

  corr_riv_csewer,      /* correlation of river flow with combined sewer flow */

  /* mode 2 */
  
  SS,                   /* mg/L     default suspended solids */
  k_SS,                 /* h-1      default suspended solids settling rate */
  f_oc,                 /* -        organic carbon fraction */

  /* mode 3 */

  v_sed_growth,         /* mm/year  sediment growth rate */
  eps_sed,              /* -        sediment porosity */
  ro_SS_sed,            /* kg/L     sediemtn solids density */

  X,                    /* mg/L     lumped biomass */
  DO,                   /* mg/L     default DO */
  BOD,                  /* mg/L     default BOD */
  pH,                   /* -        default pH */
  a_rivclass,           /* -        in-stream removal correction for this river class */
  k_light;              /* m-1      light extinction coefficient */
}
*RivClass_ParPtr;

#define INDEX_RC_SS               4
#define INDEX_RC_k_SS             5
#define INDEX_RC_DO               11
#define INDEX_RC_BOD              12

#define NR_RIVCLASS_PARAMS        16
                                  /* nr. of river class parameters in input file */


/*
 * ************************************************
 * GEO REFERENCED
 *
 * PARAMETERS
 * ************************************************
*/

/*
 * ------------------------------------------------
 * map
 * ------------------------------------------------
*/

typedef struct
{ 
  int

  upstream_one,           /* river array index of first upstream stretch */
  upstream_two,           /* river array index of second upstream stretch */
  rivclass,               /* rivclass array index of segment rivclass data */
  discharge,              /* discharge array index of segment discharge */
  emission,               /* emission array index of segment emission data */
  wwtp[NR_WWTP_CLASSES];  /* wwtp array index for each WWTP in GU */
}
*Map_Ptr;

#define MAP_SIZE      (5 + NR_WWTP_CLASSES)

/*
 * ------------------------------------------------
 * general
 * ------------------------------------------------
*/

typedef struct
{ 
  FloatType

  segment_ID,           /* -        GIS reference-code */
  segment_type,         /* -        type of segment: 
                                    nothing special, confluence, end-of-tree */
  discharge,            /* -        is there a discharge ? */
  upstream_one,         /* -        segment ID of first upstream stretch */
  upstream_two,         /* -        segment ID of second upstream stretch */
  rivclass;             /* -        river class ID */
}
*General_ParPtr;

#define NR_GEOREF_GEN_PARAMS        6       /* nr. of general parameters per seg */

/*
 * ------------------------------------------------
 * process parameters
 * ------------------------------------------------
*/

/*
 * river ------------------------------------------------
*/

typedef struct
{
  FloatType

  /* splitter fractions to next submodel(s) is not needed 
     - it was replaced by dummy_out */

  /* NOTE: FLOW MUST ALWAYS BE THE FIRST PARAMETER !!! */

  Q,            /* m3/s     flow */
  v,            /* m/s      flow velocity */
  length,       /* m        stretch length */
  depth,        /* m        river depth */
  SS,           /* mg/L     suspended solids */
  DO,           /* mg/L     dissolved oxygen */
  BOD,          /* mg/L     biol oxygen demand */
  pH,           /* -        acidity */
  X;            /* mg/L     lumped biomass */
}
*River_ParPtr;

#define INDEX_RIV_v             7
#define INDEX_RIV_depth         9
#define INDEX_RIV_SS            10
#define INDEX_RIV_DO            11
#define INDEX_RIV_BOD           12

#define NR_RIVER_PARAMS         9


/*
 * emission --------------------------------------------------------
*/

/* just before the domestic emission data, we have emission_ID */

/*
 * domestic emission ------------------------------------------------
*/

#if NR_DEST_DOM > 1

typedef struct
{
  FloatType

  /* splitter fractions to next submodel(s) */

  out[NR_SUB_DOM][NR_DEST_DOM - 1],

  /* emission */

  inhabitants,          /* cap          nr of people connected to discharge point */
  dry_weather_flow;     /* l/cap.d      domestic emission flow */

  /*
     fractionation between submodels
  
   subflow[NR_SUB_DOM][NR_STATEVARS];
  */
}
*DomesticEm_ParPtr;

#define NR_DOM_PARAMS           ( (NR_SUB_DOM)*(NR_DEST_DOM-1) + 2 )

#else

typedef struct
{
  FloatType

  /* splitter fractions to next submodel(s) not needed - just one next submodel */

  /* emission */

  inhabitants,          /* cap          nr of people connected to discharge point */
  dry_weather_flow;     /* l/cap.d      domestic emission flow */

  /* note: market data is in a separate file !!! */

  /*
     fractionation between submodels
  
   subflow[NR_SUB_DOM][NR_STATEVARS];
  */
}
*DomesticEm_ParPtr;

#define NR_DOM_PARAMS           ( 2 )

#endif



/*
 * non-domestic emission ------------------------------------------------ 
*/

#if NR_DEST_NONDOM > 1

typedef struct
{
  FloatType

  /* splitter fractions to next submodel(s) */
  out[NR_SUB_NONDOM][NR_DEST_NONDOM - 1],

  /* flow - lumped input values per non-domestic input type */
  flow[NR_SUB_NONDOM];
}
*NonDomesticEm_ParPtr;

#else

typedef struct
{
  FloatType

  /* splitter fractions to next submodel(s) not needed - just one next submodel */

  /* flow - lumped input values per non-domestic input type */
  flow[NR_SUB_NONDOM];
}
*NonDomesticEm_ParPtr;

#endif

#define NR_NONDOM_PARAMS        ((NR_SUB_NONDOM)*(NR_DEST_NONDOM - 1) + (NR_SUB_NONDOM))

/*
 * runoff ------------------------------------------------
*/

#if NR_DEST_RUNOFF > 1

typedef struct
{
  FloatType

  /* splitter fractions to next submodel(s) */
  out[NR_SUB_RUNOFF][NR_DEST_RUNOFF - 1],

  /* flow - lumped input values per runoff input type */
  flow[NR_SUB_RUNOFF];
}
*RunoffEm_ParPtr;

#else

typedef struct
{
  FloatType

  /* splitter fractions to next submodel(s) not needed - just one next submodel */

  /* flow - lumped input values per runoff input type */
  flow[NR_SUB_RUNOFF];
}
*RunoffEm_ParPtr;

#endif

#define NR_RUNOFF_PARAMS        ((NR_SUB_RUNOFF)*(NR_DEST_RUNOFF - 1) + NR_SUB_RUNOFF)

/*
 * on-site-treatment -----------------------------------------
*/

#if NR_DEST_ONSITE > 1

typedef struct
{
  FloatType

  /* splitter fractions to next submodel(s) */
  out[NR_SUB_ONSITE][NR_DEST_ONSITE - 1];
}
*OnSiteTreatment_ParPtr;

#else

typedef struct
{
  FloatType dummy;

  /* splitter fractions to next submodel(s) not needed - just one next submodel */
}
*OnSiteTreatment_ParPtr;

#endif

#define NR_ONSITE_PARAMS        ((NR_SUB_ONSITE)*(NR_DEST_ONSITE - 1))

/*
 * sewer ------------------------------------------------
*/

#if NR_DEST_SEWER > 1

typedef struct
{
  FloatType

  /* splitter fractions to next submodel(s) */
  out[NR_SUB_SEWER][NR_DEST_SEWER - 1];
}
*Sewer_ParPtr;

#else

typedef struct
{
  FloatType dummy;

  /* splitter fractions to next submodel(s) not needed - just one next submodel */
}
*Sewer_ParPtr;

#endif

#define NR_SEWER_PARAMS         ((NR_SUB_SEWER)*(NR_DEST_SEWER - 1))

/*
 * waste water treatment ------------------------------------------------
*/

#if NR_DEST_TREAT > 1

typedef struct
{
  FloatType

  /* splitter fractions to next submodel(s) */
  out[NR_SUB_TREAT][NR_DEST_TREAT - 1],

  /* matrix: WWTP class number - fraction thru this class */
  WWTP[NR_WWTP_CLASSES][2];
}
*Treatment_ParPtr;

#define BEFORE_WWTP_ID          ( (NR_SUB_TREAT)*(NR_DEST_TREAT - 1) )
#define NR_TREAT_PARAMS         ( BEFORE_WWTP_ID + NR_WWTP_CLASSES * 2 )

#else

typedef struct
{
  FloatType

  /* splitter fractions to next submodel(s) not needed
     - just one next submodel = surface water */

  /* matrix: WWTP class number - fraction thru this class */
  WWTP[NR_WWTP_CLASSES][2];
}
*Treatment_ParPtr;

#define BEFORE_WWTP_ID          0
#define NR_TREAT_PARAMS         ( NR_WWTP_CLASSES * 2 )

#endif



/*
 * small surface water ------------------------------------------------
*/

typedef struct
{
  FloatType dummy;

  /* splitter fractions to next submodel(s) not needed - just one next submodel */
}
*SmallSurf_ParPtr;

#define NR_SURFWAT_PARAMS       0

/*
 * ground water ------------------------------------------------
*/

typedef struct
{
  FloatType dummy;

  /* splitter fractions to next submodel(s) not needed - just one next submodel */
}
*GroundWat_ParPtr;

#define NR_GROUNDWAT_PARAMS     0

/*
 * ************************************************
 * WWTP
 *
 * PARAMETERS
 * ************************************************
*/

#define NR_WWTP_PARAMS        26       /* nr. of WWTP parameters in input file */

typedef struct
{ 
  FloatType

  /* WWTP ID must always be first !!! */
  
  WWTP_ID,                      /* identification of WWTP */
  mode,                         /* preferred model complexity mode */
  type,                         /* 1=primary, 2=act.sludge, 3=trickling filter */

  params[NR_WWTP_PARAMS-3];     /* all parameters */
}
*WWTP_ParPtr;

/* primary */

typedef struct
{ 
  FloatType

  WWTP_ID,              /* identification of WWTP */
  mode,			/* Intevation, 05-03-2003: Dummy to comply with spec */
  type,                 /* 1=primary, 2=act.sludge, 3=trickling filter */

  primary_present,      /* is a primary settler present ? */
  capacity_prim,        /* primary settler cap. as fract of DWF */
  h_prim,               /* m      prim. settler depth */
  v_overflow_prim,      /* m/s    prim. settler overflow rate */
  R_SS_prim_max;        /* -      max prim. settler solids removal */
}
*PrimParPtr;

/* act. sludge */

typedef struct
{ 
  FloatType

  WWTP_ID,              /* identification of WWTP */
  mode,			/* Intevation, 05-03-2003: Dummy to comply with spec */
  type,                 /* 1=primary, 2=act.sludge, 3=trickling filter */

  /* specific parameters for the PRIMARY SETTLER (if present) */

  primary_present,      /* is a primary settler present ? */
  capacity_prim,        /* primary settler cap. as fract of DWF */
  h_prim,               /* m      prim. settler depth */
  v_overflow_prim,      /* m/s    prim. settler overflow rate */
  R_SS_prim_max,        /* -      max prim. settler solids removal */

  /* specific parameters for the AS system */

    /* sec. settler */

  h_sec,                /* m      sec. settler depth */
  v_overflow_sec,       /* m/s    sec. settler overflow rate */
  R_SS_sec_max,         /* -      max sec. settler solids removal */

    /* aeration tank */

  capacity_sec,         /* aeration capacity expressed as fraction of DWF */
  nr_PE,                /* cap    nr. of people equivalents connected to this WWTP */
  SS_ML,                /* mg/L   mixed liquor SS */
  recycle,              /* -      sludge recycling ratio */
  a_biomass,            /* -      biomass activity correction */
  h_act_sludge,         /* m      mixed liquor tank depth */
  B_X,                  /* g/g.d  sludge loading rate */
  Y,                    /* g/g    biomass yield */
  DO,                   /* mg/L   actual DO in aeration tank */
  aer_type,             /*        bubble (0) or surface (1) */
  f_aerobic,            /* -      redox fractions in mixed liquor tank */
  f_anoxic,
  f_anaerobic,
  eps_N,                /* -      nitrification effciency */
  eps_DN;               /* -      denitrification effciency */
}
*ASParPtr;

/* trickling filter */

typedef struct
{ 
  FloatType

  WWTP_ID,              /* identification of WWTP */
  primary_present;      /* is a primary settler present ? */

}
*TFParPtr;


/*
 * ************************************************
 * EMISSION
 *
 * PARAMETERS
 * ************************************************
*/


typedef struct
{ 
  FloatType

  emission_ID,              /* -             identification of emission */
  consumption,              /* kg/cap.year   market data */
  nondom,                   /* g/s           nondomestic emission flux */
  runoff;                   /* g/s           runoff emission flux */
}
*EmissionParPtr;


#define NR_EMISSION_PARAMS      4       /* nr. of emission parameters in input file */

#endif

