/*
 * GREAT-ER
 *
 * CONTROL.C
 *
 * control section
 * reads input, writes output, and generates scenarios
 *
 * author =  Geert Boeije
 * state =   01.09.1998
*/

#include <time.h>
#include <string.h>
#include "init.h"
#include "data_in.h"
#include "data_out.h"
#include "mtcarlo.h"
#include "treewalk.h"
#include "usriface.h"
#include "error.h"
#include "navigate.h"
#include "util.h"
#include "model.h"
#include "control.h"

int Shot;
time_t StartTime;

#ifdef GUI_MODE
#include "extgui.h"
#endif


/*
 **********************************************************************************
 *
 * S T O C H A S T I C   L O O P 
 *
 * selection of scenarios (= Monte Carlo Simulation)
 *
 **********************************************************************************
*/

void StochasticLoop ( FloatPtr  allsegments_iface_arg,
                      FloatPtr  allsegments_summary_arg,
                      FloatPtr  allsegments_sums_arg,
                      FloatPtr  allsegments_par_arg,
                      FloatPtr  allsegments_rivclass_arg,
                      int       first_segment_index_arg )
{
  time_t t_startshots;
  FloatType flow_perc;
  int step = 1;

  /* do the Monte Carlo */

  CleanUpArray ( allsegments_sums_arg, AllSummarySize * NR_SUMMARY_SUMS );

  if ( NrMonteCarloShots > 1 )
    sprintf ( C, "Monte Carlo Simulation started for %d shots.", NrMonteCarloShots );
  else
    sprintf ( C, "Single-shot Simulation started." );
  WriteMessageInBox ( C );

  step = (int) ((float)NrMonteCarloShots / 100.0);
  step = ( step > 1 ? step : 1 );

  #ifdef GUI_MODE
  DisplayStatus = 1;
  PaintNow ();
  #endif

  t_startshots = time(NULL);


  for ( Shot=1 ; Shot <= NrMonteCarloShots ; ++Shot )
  {
    CleanUpArray ( allsegments_summary_arg,  AllSummarySize );
    CleanUpArray ( allsegments_iface_arg,    AllIfaceSize );
    CleanUpArray ( allsegments_par_arg,      AllParamSize );
    CleanUpArray ( allsegments_rivclass_arg, NR_RIVCLASS_PARAMS * NrRivClasses );

    CleanUpArray ( EmissionData,   NR_EMISSION_PARAMS * NrEmissions );
    CleanUpArray ( WWTPData,   NR_WWTP_PARAMS * NrWWTPs );

    memset ( SegmentDone, 0, NrSegments * IntSize );

    GenerateMonteCarloShot ( allsegments_par_arg, 
                             allsegments_rivclass_arg, 
                             &flow_perc );

    LogParameters ( allsegments_par_arg, 
                    allsegments_rivclass_arg, 
                    flow_perc );

    /* make some non-georeferenced model calculations... */

    ChemicalProperties ( );

    /* now, allsegments_par contains a discrete scenario !
       we will walk the river network using this scenario
       starting at the first segment                             */

    TreeWalk ( allsegments_iface_arg,
               allsegments_summary_arg,
               allsegments_par_arg,
               allsegments_rivclass_arg, 
               first_segment_index_arg );

    /* create the sums and sums of squares which are needed later
       to calculate the distribution parameters                   */

    MakeSums ( allsegments_summary_arg,
               allsegments_sums_arg     );

    #ifdef GUI_MODE
    if ( ExitCode ) return;
    #endif
    
    PercentDone ( step, (long)t_startshots, (long)time(NULL) );
  }
}

/*
 **********************************************************************************
 *
 * M A I N     C O N T R O L    U N I T  
 *
 **********************************************************************************
*/

int ControlSection (void)
{
  FloatPtr allsegments_iface;
  FloatPtr allsegments_summary;
  FloatPtr allsegments_sums;
  FloatPtr allsegments_par;
  FloatPtr allsegments_rivclass;
  FloatType first_segment_ID;
  time_t t, t_zero = time(NULL);
  long tot_alloc;
  int first;
  int err[1];


  /*
   ------------------------------------------
   INITIALIZING
   ------------------------------------------
  */

  if (!InstallErrorFile ())
    return 0;

  if (!(ObtainControlParams ( &first_segment_ID )))
    return 0;

  /* now we have: all control parans  */

  #ifdef GUI_MODE
  ArrangeMainWindowVisibility ();
  DisplayStatus = 0;
  PaintNow ();
  #endif

  if (!InstallOutputFiles ())
    return 0;

  /* Initialize */

  AllIfaceSize = NrSegments * NR_IFACE_TERMS * NR_STATEVARS;
  AllParamSize = NrSegmentsOnlyRiver * NrParamsOnlyRiver 
                 + NrSegmentsDischarge * NrParamsDischarge;
  AllSummarySize = NrSegments * NR_SUMMARY_TERMS * NR_SUMMARY_VARS;

  if (!AllocateGlobals ( 1, &tot_alloc )) return 0;
  
  if (!(ObtainDistriTypes ()))
    return 0;

  CalculateParametersArrayOffset ();
  CalculateStateVarsArrayOffset ();
  CalculateAmatrixOffset ();
  CalculateDistriParIndex ();

  if (!AllocateGlobals ( 2, &tot_alloc )) return 0;

  if (!(AllocateLocals ( &allsegments_iface,
                         &allsegments_summary,
                         &allsegments_sums,
                         &allsegments_par,
                         &allsegments_rivclass,
                         &tot_alloc        )    )) return 0;

  LogControlParams ( first_segment_ID, tot_alloc );

  /* no we read the input files !!! */

  if (!ObtainInputData ()) return 0;

  if (!ObtainExceptions()) return 0;

  /* last offset calculation - based on data in file which was just read */

  CalculateSegmentParametersOffset ();  

  /* replace Q95 low flow values by standard deviations */

  InitializeFlowTypeDistrib ();

  /* initialize tree walk map */

  if (!InitializeTree ( )) return 0;

  if ( (first = SegmentNumber ( -1, first_segment_ID, _UP_ )) == -1 )
  {
    GenerateErrorMessage (505, &(first_segment_ID) );
    return 0;
  }

  WriteMessageInBox ("Reading of Data Files and Initialization was successful.");


  /*
   -------------------------------------
   run the stochastic loop (Monte Carlo simulation)
   -------------------------------------
  */

  StochasticLoop ( allsegments_iface,
                   allsegments_summary,
                   allsegments_sums,
                   allsegments_par,
                   allsegments_rivclass,
                   first );

  #ifdef GUI_MODE
  if ( ExitCode )
  {
    err[0] = Progress;
    GenerateErrorMessage ( 999, err);
    goto exit_steps;    
  }
  PercentDone ( NrMonteCarloShots, 0, 0 );
  #endif

  /*
   -------------------------------------
   calculate distributions of results
   -------------------------------------
  */

  CalculateDistribution ( allsegments_sums );

  WriteOutputFile ( allsegments_par );

  /* simulation finished */


  AllFinished = 1;


  if ( ErrorFree )
  {
    t = time(NULL) - t_zero;
    sprintf ( C, "Simulation ended successfully.\n"
                 "(simulation time = %d second%s)", (int)t, (int)t == 1 ? "" : "s" );
  }
  else
    sprintf ( C, "Simulation ended - errors were encountered !");

  WriteMessageInBox ( C );

  strcat ( C, "\n\n");
  WriteLog ( C );

  #ifndef GUI_MODE
  if ( Visible )
    printf ("\n\n");
  #endif

  if ( LogSummaryResults )
    LogSummaryDistribs ( allsegments_par );

  /*
   -------------------------------------
   final actions
   -------------------------------------
  */

  #ifdef GUI_MODE
  exit_steps:
  #endif

  FreeLocals ( allsegments_iface,
               allsegments_summary,
               allsegments_sums,
               allsegments_par,
               allsegments_rivclass );

  FreeGlobals ();

  CloseOutputFiles ();

  if ( ErrorFree )
  {
    remove ( ErrorFileName );
    return 1;
  }
  else
    return 0;
}

/*
 **********************************************************************************
 *
 * S I M U L A T I O N    K I C K O F F
 *
 **********************************************************************************
*/


int Simul ( void )
{
  int result;

  StartTime = time(NULL);

  Initialize ();

  result = ControlSection ();

  #ifdef GUI_MODE
  if ( ! result || ExitCode )
    WriteMessageInBox ("Simulation failed.");
  #else
  if ( ! result )
    WriteMessageInBox ("Simulation failed.");
  #endif

  return result;
}
