/*
 * GREAT-ER
 *
 * MODUTIL.C
 *
 * model section - utilities
 *
 * author =  Geert Boeije
 *
 * changes by Intevation, http://intevation.net
 * state =   16.02.2000
 *
 * Intevation: replaced IN and OUT by IN__ and OUT__
*/

#include "init.h"
#include "model.h"
#include "navigate.h"
#include "modutil.h"

/*
 * **************************
 *
 * UTILITY ROUTINES FOR  MODELS
 *
 * **************************
*/

/*
 * ---------------------------------------------------------------------
 * segment inflow
 * ---------------------------------------------------------------------
*/

void InflowIntoSegment ( FloatPtr segment_iface_arg,
                         FloatPtr river_par_arg )
{
  River_ParPtr p = (River_ParPtr) river_par_arg;  

  B [ oX(_PHI_,RIVER,IN__,DEFAULT) ] = segment_iface_arg[ _PHI_*NR_IFACE_TERMS + IN__ ];

  B [ oX(_Q_,RIVER,IN__,DEFAULT) ] = p->Q;
}

/*
 * ---------------------------------------------------------------------
 * put outflow fractions from each submodel in a process into the A matrix
 * on the row of the destination submodels
 * ---------------------------------------------------------------------
*/

void DestinationSplitter ( int      process_arg,
                           FloatPtr fract_arg,
                           int      gw_mode_arg )
{
  FloatType destination_fract;
  int i, j, k;
  int position, row, col, first_col, from;

  from = ( ModelClass[process_arg] == EMISSION ? EM : OUT__ );

  /* column in the A matrix points to OUT or EM section of 'from' statevariable */
  
  first_col = oX (0, process_arg, from, 0); 

  for ( i=0 ; i<NrSub[process_arg] ; ++i )
  {
    col = first_col + i;

    for ( j=0 ; j<NrDestinations[process_arg] ; ++j )
    {
      /* row in the A matrix points to IN section of 'to' state variable */

      row = oX ( 0, Destination[process_arg][j][0], IN__, Destination[process_arg][j][1]);

      destination_fract = fract_arg[ i*NrDestinations[process_arg] + j];

      /* if groundwater leakage BEFORE submodel then - for the  groundwater leakage 
         process - 'from' points to IN state variable instead of the OUT state variable
         (we need the value which is UNMODIFIED by the submodel !)
         and, for the other destinations the fractions have to be corrected */
      
      if ( gw_mode_arg && j==DestNrBypass[process_arg] )
        col = oX (0, process_arg, IN__, i);
      else if ( gw_mode_arg && destination_fract )
        destination_fract /= 
          ( 1 - fract_arg[ i*NrDestinations[process_arg] + DestNrBypass[process_arg] ] );

      position = Acoordinates (row, col);

      for ( k=0 ; k<NR_STATEVARS ; ++k )
        A[ AsubMatrixSize*k + position ] = destination_fract;

      /* if we changed col we have to change it back to its normal value */

      if ( gw_mode_arg && j==DestNrBypass[process_arg] )
        col = first_col + i;
    }
  }
}

/*
 * ---------------------------------------------------------------------
 * submatrix calculations
 *
 * subarray X[firstrow..torow-1] is calculated
 * ---------------------------------------------------------------------
*/

void PerformSubmatrixOperation ( int      from_row_arg,
                                 int      nr_rows_arg,
                                 int      process_arg,
                                 int      terminal_arg )
{
  int i, j, k;
  int kstart = 0, kstop = 0;
  int submatrix_offset=0, A_position=0;
  int statevar_offset=0, X_position=0, X_position_in=0;

  /* perform the model calculations for:
     - the current process OUTPUT
     - the next process INPUT */

  /* general model: X = A * X + B */

  /* here: do calculation only for current subblock

     Xsub  = Asub1 * Xsub1 + Bsub
     (nx1) = (nxm) * (mx1) + (nx1)

       n = nr. of X values in selected subarray
       m = cumulative nr. of X values until and including current subarray
  */

  for ( i=0 ; i<NR_STATEVARS ; ++i )
  {
    if ( ModelClass[process_arg] == TRANSTREAT )
    {
      if (terminal_arg == IN__)
      {
        kstart = 0;
        kstop = from_row_arg - 1;
      }
      else
      {
        kstart = from_row_arg - nr_rows_arg;
        kstop = kstart;
      }
    }

    for ( j=from_row_arg ; j<from_row_arg + nr_rows_arg ; ++j )
    {
      X_position = statevar_offset + j;

      if ( ModelClass[process_arg] == TRANSTREAT )
      {
        /* X = A*X . . . */

        for ( k=kstart ; k<=kstop ; ++k )
        {
          A_position = submatrix_offset + Acoordinates(j,k);
          X_position_in = statevar_offset + k;

          if ( X [X_position_in] && A [A_position] )
            X [X_position] += A[A_position] * X [X_position_in];
        }

        if (terminal_arg == OUT__)
        {
          ++kstart;
          ++kstop;
        }
      }

      /* X = . . . + B */

      if ( B [X_position] )
        X [X_position] += B [X_position];
    }

    submatrix_offset += AsubMatrixSize;
    statevar_offset += OneStateVarSize;
  }
}

/*
 * ---------------------------------------------------------------------
 * interface variables
 * ---------------------------------------------------------------------
*/

void CalculateInterfaceVars ( FloatPtr segment_iface_arg )
{
  int i;

  /* input = was already filled in before the simulation of this segment
     output = output from last process
  */

  for ( i=0 ; i<NR_STATEVARS ; ++i )
    segment_iface_arg[ i*NR_IFACE_TERMS + OUT__ ] = X[ oX(i,LAST,OUT__,DEFAULT) ];
}

/*
 * ---------------------------------------------------------------------
 * summary variables
 * ---------------------------------------------------------------------
*/

void CalculateSummaryVars ( FloatPtr segment_summary_arg,
                            int      a_discharge_arg,
                            FloatType stretch_length_arg )
{
  FloatType flow;

  /* the only summary variable here is C (chemical concentration) */

  /* if the stretch is virtual then no summary ! */

  if ( stretch_length_arg == 0.0 )
  {
    segment_summary_arg[ oS(_C_,RIV_MAX) ] = -1;
    segment_summary_arg[ oS(_C_,RIV_OUT) ] = -1;
    segment_summary_arg[ oS(_C_,RIV_INT) ] = -1;

    if ( a_discharge_arg )
    {
      segment_summary_arg[ oS(_C_,CAT_MAX) ] = -1;
      segment_summary_arg[ oS(_C_,CAT_OUT) ] = -1;
      segment_summary_arg[ oS(_C_,CAT_INT) ] = -1;
    }
  }

  else

  /* it's a real stretch */

  {

    /* RIVER */
    /*-------*/

    /* maximal = the value at mixing with discharge */

    flow = X[ oX(_Q_,RIVER,IN__,DEFAULT) ];

    segment_summary_arg[ oS(_C_,RIV_MAX) ] 
    = ( flow > 0 ? X[ oX(_PHI_,RIVER,IN__,DEFAULT) ] / flow : -1 );
  
    /* output = output from river process */
  
    flow = X[ oX(_Q_,RIVER,OUT__,DEFAULT) ];
  
    segment_summary_arg[ oS(_C_,RIV_OUT) ] 
    = ( flow > 0 ? X[ oX(_PHI_,RIVER,OUT__,DEFAULT) ] / flow : -1 );
  
    /* internal = an average value for this segment - calculated somewhere in MODELS.C */
  
    flow = RiverInternal[_Q_];
  
    segment_summary_arg[ oS(_C_,RIV_INT) ]
    = ( flow > 0 ? RiverInternal[_PHI_] / flow : -1 );
  
    
    /* CATCHMENT */
    /*-----------*/

    /* originally, PEC_catchment was intended to be used for so-called
       small surface waters; however this appeared to be not necessary 
       in the GREAT-ER concept.  Now, PEC_catchment is used to store WWTP
       influent and effluent concentrations */
  
    if ( a_discharge_arg )
    {
      /* maximal = WWTP influent */
      /* note that we look at BOTH the TREATED and the UNTREATED fraction */
      /* hence PEC = weighted average by flow of conc. which enters the treated 
         and the untreated submodule of the WWTP model */
  
      flow = X[ oX(_Q_,TREAT,IN__,TREATED) ] + X[ oX(_Q_,TREAT,IN__,NONE) ];

      if ( flow <= 0 )
        segment_summary_arg[ oS(_C_,CAT_MAX) ] = -1;        
      else
        segment_summary_arg[ oS(_C_,CAT_MAX) ]
        = ( X[ oX(_PHI_,TREAT,IN__,TREATED) ] + X[ oX(_PHI_,TREAT,IN__,NONE) ] ) / flow;


      /* output = WWTP effluent */
      /* note that we look at BOTH the TREATED and the UNTREATED fraction */
      /* hence PEC = weighted average by flow of conc. which enters the treated 
         and the untreated submodule of the WWTP model */

      flow = X[ oX(_Q_,TREAT,OUT__,TREATED) ] + X[ oX(_Q_,TREAT,OUT__,NONE) ];

      if ( flow <= 0 )
        segment_summary_arg[ oS(_C_,CAT_OUT) ] = -1;        
      else
        segment_summary_arg[ oS(_C_,CAT_OUT) ]
        = ( X[ oX(_PHI_,TREAT,OUT__,TREATED) ] + X[ oX(_PHI_,TREAT,OUT__,NONE) ] ) / flow;

      /* internal = not used */
  
      segment_summary_arg[ oS(_C_,CAT_INT) ] = -1;
    }
    else
    {
      segment_summary_arg[ oS(_C_,CAT_MAX) ] = -1;
      segment_summary_arg[ oS(_C_,CAT_OUT) ] = -1;
      segment_summary_arg[ oS(_C_,CAT_INT) ] = -1;
    }
  }

}

/*
 * ---------------------------------------------------------------------
 * make the out matrices
 * ---------------------------------------------------------------------
*/

void MakeOutMoreMatrix ( FloatPtr out_arg,
                         FloatPtr p_out_arg,
                         int nr_sub_arg,
                         int nr_dest_arg )
{
  int i, j;
  FloatType sum, temp;

  for ( i=0 ; i<nr_sub_arg ; ++i )
  {
    sum = 0;

    /* in p_out_arg, we have destinations 1..end for each submodel */

    for ( j=0 ; j<(nr_dest_arg - 1) ; ++j )
    {
      temp = p_out_arg[ i*(nr_dest_arg - 1) + j ];
      sum += temp;
      out_arg[ i*nr_dest_arg + j + 1] = temp;
    }

    /* what if sum > 1   then we normalize to one and sum is set to 1 */

    if ( sum > 1.0 )
    {
      for ( j=0 ; j<(nr_dest_arg - 1) ; ++j )
        out_arg[ i*nr_dest_arg + j + 1] /= sum;

      sum = 1;
    }

    /* destination 0 gets the leftovers */

    out_arg[ i*nr_dest_arg ] = 1 - sum;
  }
}

void MakeOutOneMatrix ( FloatPtr out_arg,
                        int nr_sub_arg )
{
  int i;

  for ( i=0 ; i<nr_sub_arg ; ++i )
    out_arg[ i ] = 1;
}

