/*
 * GREAT-ER
 *
 * R_RIVER.C
 *
 * fate models - calculation of removal rates
 *               for river 
 *
 * author =  Geert Boeije
 * state =   07.01.1999
*/

#include <math.h>
#include "init.h"
#include "model.h"
#include "r_river.h"


/*
 * **********************************************************************
 * river hydrolysis model
 * **********************************************************************
*/

FloatType RiverHydrolysis ( FloatType pH_arg )
{
  Chemical_ParPtr chem = ChemicalParams;
  FloatType h;
  
  if ( pH_arg != -1 )
    h =   chem->k_hydr_n 
        + chem->k_hydr_b * pow ( 10, pH_arg-14.0 )
        + chem->k_hydr_a * pow ( 10, -pH_arg );
  else
    h = chem->k_hydr_n;

  return h;
}

/*
 * **********************************************************************
 * river photolysis model
 * **********************************************************************
*/

FloatType RiverPhotolysis ( FloatType depth_arg )
{
  Chemical_ParPtr chem = ChemicalParams;
  RivClass_ParPtr rc   = (RivClass_ParPtr) SegmentRivClass;
  FloatType light_ext, ph;
  
  if ( chem->k_photo != -1 && rc->k_light != -1 )
  {
    light_ext = rc->k_light * depth_arg;
    ph = chem->k_photo * ( 1 - exp(-light_ext) ) / light_ext;
  }
  else
    ph = 0;

  return ph;
}

/*
 * **********************************************************************
 * river biodegradation model
 * **********************************************************************
*/

FloatType RiverBiodeg ( FloatType DO_arg,
                        FloatType X_arg,
                        FloatType f_dissolved_arg,
                        FloatType f_sorbed_arg )
{
  Chemical_ParPtr chem = ChemicalParams;
  NonGeoRef_ParPtr ngr = NonGeoRefParams;
/*  RivClass_ParPtr rc   = (RivClass_ParPtr) SegmentRivClass;*/
  FloatType bd, a_DO, a_temp;
  
  if ( chem->k_std_biodeg  != -1 )
  {
    a_DO =   DO_arg / ( chem->K_DO + DO_arg )  
           + ( chem->K_DO / ( chem->K_DO + DO_arg ) ) * chem->a_anaerobic;

    a_temp = pow ( chem->Q10, (ngr->temp_water-20.0) / 10.0 );

    bd =     /* double first order -> pseudo first order */
             chem->k_std_biodeg * X_arg     

             /* corrected for river environment */
           * chem->a_river                  

             /* corrected for sorbed vs. dissolved phase biodegradation */
           * ( f_dissolved_arg + f_sorbed_arg * chem->a_sorbed )

             /* corrected for the redox potential */
           * a_DO

             /* corrected for temperature */
           * a_temp;
  }
  else
    bd = 0;

  return bd;
}

/*
 * **********************************************************************
 * river sedimentation model
 * **********************************************************************
*/

/* maybe this can be refined later on ??? */

FloatType RiverSedimentation ( FloatType depth_arg,
                               FloatType SS_arg,
                               FloatType f_sorbed_arg )
{
  RivClass_ParPtr rc   = (RivClass_ParPtr) SegmentRivClass;
  FloatType sedrate, v_sed, s;

  v_sed =   rc->v_sed_growth * 3.171e-11 
          * 1e6 * rc->ro_SS_sed 
          * (1-rc->eps_sed)
          / SS_arg;

  sedrate = v_sed / depth_arg;

  s = f_sorbed_arg * sedrate;

  return s;
}


/*
 * **********************************************************************
 * river volatilization model
 * **********************************************************************
*/

FloatType RiverVolatilization ( FloatType depth_arg,
                                FloatType velocity_arg,
                                FloatType pH_arg,
                                FloatType f_dissolved_arg )
{
  RivClass_ParPtr rc   = (RivClass_ParPtr) SegmentRivClass;
  NonGeoRef_ParPtr ngr = NonGeoRefParams;
  Chemical_ParPtr chem = ChemicalParams;

  FloatType v = 0;

  FloatType SC_g = 0;
  FloatType SC_l = 0;
  FloatType v_shear = 0;
  FloatType alpha_shear = 0;
  FloatType beta_shear = 0;
  FloatType k_l = 0;
  FloatType k_g = 0;
  FloatType v_fr = 0;
  FloatType F = 0;
  FloatType K_H;
  FloatType temp;

  switch ( (int)rc->lake )
  {
    /* ------------*/
    /* it's a LAKE */  
    /* ------------*/

    case 1:     

      /* Mackay and Yeun approach */

      /* Schmidt numbers */
      /* SC_g = 0.0000014 / ( 2.22 * sqrt ( 18.0 / chem->MM ) ); */
      /* SC_l = 0.000001  / ( 0.0001728 * sqrt ( 32.0 / chem->MM ) );*/

      temp = sqrt ( chem->MM );
      SC_g = 0.0000001486410651143 * temp ;
      SC_l = 0.001023013282967 * temp ;

      /* shear velocity + coeffs. acc. to Monim and Yaglom */
      v_shear = 0.0359 * pow ( ngr->wind_speed, 0.93 );

      if ( v_shear < 0.3 )
      {
        alpha_shear = 51.84;
        beta_shear = 2.2;
      }
      else
      {
        alpha_shear = 12.27;
        beta_shear = 1.0;
      }

      /* conductances gaseous and liquid film */

      k_g = 3.6 + 166.32 * v_shear / pow ( SC_g, 0.66666667 );
      k_l = 0.0036 + alpha_shear * pow ( v_shear, beta_shear ) / sqrt ( SC_l);

      break;

    /* -------------*/
    /* it's a RIVER */
    /* -------------*/

    case 0:
    default:

      /* Southworth approach */

      /* friction velocity */
      /* it is assumed that z0 = 0.001 m */
      /* v_fr = ngr->wind_speed * ( log10 (0.1/ngr->z0) / log10 (10/ngr->z0) );*/

      v_fr = 0.5 * ngr->wind_speed;

      /* correction factor */
      F = ( v_fr < 1.9 ? 1 : exp( 0.526 * (v_fr-1.9) ) );

      /* conductances gaseous and liquid film */

      /* k_l =   0.2351 * F * pow (velocity,0.969) / pow (depth, 0.673) 
               * sqrt ( 32 / chem->MM ); */
      /* k_g =   11.37 * ( velocity + v_fr ) 
               * sqrt ( 18 / chem->MM ); */

      /* note: k_g and k_l are in m/h while velocities are in m/s */

      temp = sqrt ( chem->MM );

      k_l =   1.329926434056 * F 
            * pow (velocity_arg, 0.969) / pow (depth_arg, 0.673) 
            / temp;

      k_g =   48.23882461255 * ( velocity_arg + v_fr ) / temp;

      break;
  }

  /* ------------------------------- */
  /* dimensionless Henry coefficient */
  /* ------------------------------- */

  /* correction of Henry coeff. for pH */

  if ( chem->a && pH_arg != chem->pKa )
    K_H = CP.K_H / ( 1 + pow (10, chem->a*(pH_arg - chem->pKa)) );
  else
    K_H = CP.K_H;

  /* ------------------- */
  /* volatilization rate */
  /* ------------------- */

  v = f_dissolved_arg * (1 / depth_arg) * (1 / (1/k_l + 1/(k_g * K_H)));

  return v;
}

/*
 * **********************************************************************
 * river removal model
 * **********************************************************************
*/

FloatType RiverRemoval ( FloatPtr  process_x_in_arg,
                         FloatPtr  process_par_arg,
                         FloatPtr  general_par_arg,
                         FloatType leak_dummy_arg,
                         int       submodel_dummy_arg )
{
  River_ParPtr p       = (River_ParPtr) process_par_arg;
  RivClass_ParPtr rc   = (RivClass_ParPtr) SegmentRivClass;
  Chemical_ParPtr chem = ChemicalParams;

  FloatType r = 0;
  FloatType e_power = 0;
  FloatType travel_time = 0;
  FloatType decay_rate = 0;
  FloatType Kd_river;
  FloatType f_sorbed = 0;
  FloatType f_dissolved = 0;
  FloatType k_sed = 0;
  FloatType k_vol = 0;
  FloatType k_deg = 0;
  FloatType k_biodeg = 0;
  FloatType k_photodeg = 0;
  FloatType k_hydr = 0;
  FloatType PHI_start = 0;
  FloatType SS = 0;
  FloatType pH = 0;
  FloatType DO = 0;
  FloatType X = 0;
  int complexity_mode, i;
  int no_exception_found = 1;

  /**************************/
  /* which mode do we use ? */
  /**************************/

  if ( ImposeComplexityModeRiver || rc->mode == -1 )
  {
    if ( ComplexityModeRiver != -1 )
      complexity_mode = ComplexityModeRiver;
    else
      complexity_mode = 1; /* if nothing is specified, take mode 1 as default */
  }
  else
    complexity_mode = rc->mode;

  /***********************/
  /* zero-length stretch */
  /***********************/

  if ( p->length == 0.0 )
  {
    RiverInternal[_PHI_] = process_x_in_arg[ _PHI_*OneStateVarSize ];
    RiverInternal[_Q_] = p->Q;
    return 0;
  }

  /******************************/
  /* calculation of travel time */
  /******************************/

  switch ( (int)rc->lake )
  {
    case 1:     /* it's a LAKE */

      /* p->length is not length but VOLUME for a LAKE */

      if ( p->Q )
      {
        travel_time = p->length / p->Q;       /* in seconds */
        travel_time /= 3600;                  /* converted to hours */
      }
      else
        travel_time = -1;                     /* travel time = infinity */

      break;

    case 0:     /* it's a RIVER */
    default:

      /* travel time */  
  
      if ( p->v > 0 )
      {
        travel_time = (p->length / p->v ); /* in seconds */
        travel_time /= 3600;                  /* converted to hours */
      }
      else
        travel_time = -1;                     /* travel time = infinity */

      break;
  }

  /*****************************/
  /* calculation of decay rate */
  /*****************************/

  /* do we have site-specific data ? */

  if ( UseExceptions && NrRiverExceptions )
  {
    for ( i=0 ; i<NrRiverExceptions ; ++i )
    {
      if ( RiverExceptions[ i*2 ] == ThisSegmentID )
      {
        decay_rate = RiverExceptions[ i*2 + 1 ];
        no_exception_found = 0;
      }
    }
  }

  if ( no_exception_found )
  {
    /* Complexity Modes 2 and 3 */  
  
    if ( complexity_mode > 1 )
    {  
      /* take river class values for SS, DO, pH if not given for river stretch */
  
      if ( complexity_mode == 3 )
      {
        if ( (pH = p->pH) == -1 ) pH = rc->pH;
        if ( (DO = p->DO) == -1 ) DO = rc->DO;
        if ( (X = p->X) == -1 )   X = rc->X;
      }
  
      /* SS are NOT given for this river stretch */
  
      if ( p->SS == -1 ) 

        /* set SS to default background value fo this river class */  
        SS = rc->SS;
  
      else

        /* SS are given for this individual stretch */
        SS = p->SS;
  
      /* calculate sorbed vs. dissolved */
  
      if ( SS != -1 )
      {
        if (chem->Kd_river == -1 )
          Kd_river = rc->f_oc * chem->K_ow;
        else
          Kd_river = chem->Kd_river;
  
        f_dissolved = 1 / (1 + Kd_river * 0.000001 * SS );
        f_sorbed = 1 - f_dissolved;
      }
      else
      {
        f_dissolved = 1;
        f_sorbed = 0;
      }
    }
  
    switch ( complexity_mode )
    {
      /*-------------------*/
      /* Complexity Mode 1 */
      /*-------------------*/
  
      case 1:
  
        decay_rate = ChemicalParams->k_instream;
        break;
  
      /*-------------------*/
      /* Complexity Mode 2 */  
      /*-------------------*/
  
      case 2:
      
        decay_rate = chem->k_deg + f_dissolved * chem->k_vol + f_sorbed * rc->k_SS;
        break;
  
      /*-------------------*/  
      /* Complexity Mode 3 */  
      /*-------------------*/
  
      case 3:
  
        /* hydrolysis **********************************/
  
        if ( chem->m3_deg_hydr )
          k_hydr = RiverHydrolysis ( pH );
        else
          k_hydr = 0;
  
        /* photolysis ****************************************/
  
        if ( chem->m3_deg_photo )
          k_photodeg = RiverPhotolysis ( p->depth );
        else
          k_photodeg = 0;
  
        /* biodegradation ************************************/
  
        if ( chem->m3_deg_bio )
          k_biodeg = RiverBiodeg ( DO, X, f_dissolved, f_sorbed );
        else
          k_biodeg = 0;
  
        /* lumped degradation ***********************************/
      
        k_deg = k_photodeg + k_hydr + k_biodeg;
                   
  
        /* sedimentation ****************************************/
  
        if ( chem->m3_sed )
          k_sed = RiverSedimentation ( p->depth, SS, f_sorbed );
        else
          k_sed = 0;
  
  
        /* volatilisation ****************************************/
  
        if ( chem->m3_vol != -1 )
          k_vol = RiverVolatilization ( p->depth, p->v, pH, f_dissolved );
        else
          k_vol = 0;
  
        /* lumped **************************************************/
  
        decay_rate = k_deg + k_sed + k_vol;
  
        break;
  
      default: 
  
        decay_rate = 0;
        break;
    }

    /* correction for this river class */
    decay_rate *= rc->a_rivclass; 
  }
  

  /**************************/
  /* calculation of removal */
  /**************************/

  if ( travel_time != -1 )
  {
    e_power = travel_time * decay_rate;

    switch ( complexity_mode )
    {
      /* case 1 and 2: no solubility correction */
    
      case 1:
      case 2:

        r = 1 - exp( - e_power );

        if ( r>0 )
          RiverInternal[_PHI_]
            = process_x_in_arg[ _PHI_*OneStateVarSize ] / e_power * r;
        else
          RiverInternal[_PHI_] 
            = process_x_in_arg[ _PHI_*OneStateVarSize ];

        break;

      /* case 3: solubility correction included */

      case 3:

        /* C above solubility limit */

        if ( process_x_in_arg[ _PHI_*OneStateVarSize ] / p->Q > chem->solubility )
        {
          /* use solubility limit as starting point */
          PHI_start = chem->solubility * p->Q;

          /* calculate removal by comparing end value to orig. PHI 
             (ie before correction solubility) */
          r = 1 - (PHI_start * exp(-e_power)) 
                  / process_x_in_arg[ _PHI_*OneStateVarSize ];

          /* average of 1st order decay - after solub. corr. */

          if ( r>0 )
            RiverInternal[_PHI_] = PHI_start / e_power * r;
          else
            RiverInternal[_PHI_] = PHI_start ;

        }
        else

        /* everything soluble */        
        {
          r = 1 - exp( - e_power );

          /* average of 1st order decay */

          if ( r>0 )
            RiverInternal[_PHI_]
              = process_x_in_arg[ _PHI_*OneStateVarSize ] / e_power * r;
          else
            RiverInternal[_PHI_] 
              = process_x_in_arg[ _PHI_*OneStateVarSize ];

        }
        break;
    }
  }

  else
  {
    /* 'all or nothing' because HRT = infinity */
  
    r = ( decay_rate > 0 ? 1 : 0 );

    RiverInternal[_PHI_] = ( r == 1 ? 0 : process_x_in_arg[_PHI_] );
  }

  RiverInternal[_Q_] = p->Q;

  return r;
}


