/*
 * GREAT-ER
 *
 * R_WWTP.C
 *
 * fate models - calculation of removal rates
 *               for WWTPs
 *
 * author =  Geert Boeije
 * state =   07.01.1999
*/

#include <math.h>
#include "init.h"
#include "model.h"
#include "modcalc.h"
#include "r_wwtp.h"

FloatType PrimaryEffluentSS;
FloatType fTreatedPrimary;
FloatType PurePrimaryRemoval;
int CurrentWWTPComplMode;

/*
 * ----------------------------------------------------------------------
 * Treated Fraction 
 * ----------------------------------------------------------------------
*/


FloatType FractionTreated ( FloatType cap_arg,
                            FloatType dwf_arg,
                            FloatType q_in_arg )
{
  FloatType f;

  if ( dwf_arg )
  {
    f = cap_arg / ( q_in_arg / dwf_arg );
    f = ( f > 1 ? 1 : f );
  }
  else
    f = 1;

  return f;
}

/*
 * ----------------------------------------------------------------------
 * Effluent Solids
 * ----------------------------------------------------------------------
*/

FloatType SettlerEfficiency ( FloatType r_SS_max_arg,
                              FloatType alpha_arg,
                              FloatType q_arg,
                              FloatType dwf_arg,
                              FloatType ss_arg )
{
  FloatType r;  

  if ( dwf_arg )
  {
    r = r_SS_max_arg - alpha_arg * q_arg / dwf_arg;    
    return r;
  }
  else
    return 0;

}


/*
 * **********************************************************************
 *
 * primary settler
 *
 * **********************************************************************
*/

/*
 * ------------------------------------------
 * Modified SimpleTreat 3.0 / Primary Settler
 * 3-box model
 * ------------------------------------------
*/

FloatType PrimarySimpleTreat ( FloatPtr wwtp_par_arg,
                               FloatType q_in_arg,
                               FloatType phi_in_arg,
                               FloatType dwf_arg,
                               FloatType* f_dissolved_arg,
                               FloatType* f_sorbed_arg,
                               FloatType R_SS_prim_arg )
{
  PrimParPtr p = (PrimParPtr) wwtp_par_arg;
  NonGeoRef_ParPtr ngr = NonGeoRefParams;

  int i,j;

  FloatType r;
  FloatType a_prim;
  FloatType v_prim;
  FloatType ss_eff;
  FloatType c_influent, c_effluent;
  FloatType diff;
  FloatType sum;
  FloatType v[5];
  FloatType k[5];
  FloatType adv[5][5];
  FloatType xch[5][5];
  FloatType conc[5];
  FloatType phi[5];
  FloatType _coef[4][4];
  FloatPtr  _c = &(conc[1]);
  FloatPtr  _phi = &(phi[1]);

  /* initialize */

  for ( i=0 ; i<5 ; i++ )
  {
    v[i] = 0;
    k[i] = 0;
    conc[i] = 0;
    phi[i] = 0;

    for ( j=0 ; j<5 ; j++ )
    {
      adv[i][j] = 0;
      xch[i][j] = 0;
    }
  }

  for ( i=0 ; i<4 ; i++ )
    for ( j=0 ; j<4 ; _coef[i][j++] = 0 );
  
  /*
   * --------------------------------------------- *
   * estimation of design and operation parameters *
   * --------------------------------------------- *
  */

  /* effluent solids */

  ss_eff = ngr->SS_sewage * ( 1.0 - R_SS_prim_arg );

  /* influent concentration */

  if ( q_in_arg > 0 )
    c_influent = phi_in_arg / q_in_arg ;
  else
    return 0;

  /* influent partitioning (assuming total equilibrium in sewer) */

  *f_dissolved_arg = 1.0 / ( 1.0 + 1e-6 * ngr->SS_sewage * CP.Kd_sewage );
  *f_sorbed_arg    = 1.0 - *f_dissolved_arg;

  /*
   * PRIM. SETTLER
   *
  */

  a_prim = dwf_arg / p->v_overflow_prim;                              /* m2 */
  v_prim = a_prim * p->h_prim;                                        /* m3 */


  /*
   * -------------------- *
   * influent calculation *
   * -------------------- *
  */

  phi[2] = *f_dissolved_arg * phi_in_arg;
  phi[3] = *f_sorbed_arg * phi_in_arg;

  /*
   * -------------------- *
   * advective transport  *
   * -------------------- *
  */

  /* air advection */

  adv[0][1] = ngr->h_air * ngr->wind_speed * sqrt( a_prim );
  adv[1][0] = adv[0][1];

  /* water volume flow rates */

  adv[0][2] = q_in_arg;
  adv[2][0] = adv[0][2];

  /* solids volume flow rate */

  adv[0][3] = ngr->SS_sewage * 1e-6 * ( 1.0 / ngr->ro_SS_sewage ) * q_in_arg;
  adv[3][4] = R_SS_prim_arg * adv[0][3];
  adv[3][0] = ( 1.0 - R_SS_prim_arg ) * adv[0][3];
  adv[4][0] = adv[3][4];


  /*
   * -------------------- *
   * diffusive exchange   *
   * -------------------- *
  */

  /* box volumes */
  /* ----------- */

  v[1] = a_prim * ngr->h_air;
  v[2] = v_prim;
  v[3] = v_prim * ngr->SS_sewage * 1e-6 * ( 1.0 / ngr->ro_SS_sewage );
  /* v[4] need not be known ! */

  /* media volume flow rates */
  /* ----------------------- */

  /**** sludge / water */

  diff = ngr->k_sorb_prim / ( 1.0/(v[2]*FugacityWater) + 1.0/(v[3]*FugacityPrimSludge) );
  xch[2][3] = diff / FugacityWater;
  xch[3][2] = diff / FugacityPrimSludge;

  /**** air / water */

  diff = a_prim / ( 1.0/(ngr->K_air*FugacityAir) + 1.0/(ngr->K_water*FugacityWater) );
  xch[1][2] = diff / FugacityAir;
  xch[2][1] = diff / FugacityWater;


  /*
   * --------------------------------- *
   * fill in and solve the matrices    *
   * --------------------------------- *
  */

  /* note: _coef[0][0] corresponds with adv[1][1] etc. */

  for ( i=1 ; i<5 ; i++ )
    for ( j=1 ; j<5 ; j++ )
      if ( i != j)
        _coef[i-1][j-1] = - ( adv[j][i] + xch[j][i] );

  for ( i=1 ; i<5 ; i++ )
  {
    for ( sum=0, j=1 ; j<5 ; j++ )
      sum += ( adv[i][j] + xch[i][j] );

    _coef[i-1][i-1] = k[i]*v[i] + sum + adv[i][0];
  }

  /* solver: _coef * _c = _phi */
  /* linear system A * X = B  ->  arguments = A, B, X and n (with A - n x n matrix) */

  SolveLinearSystem ( (FloatType*)_coef, (FloatType*)_phi, (FloatType*)_c, 4);

  c_effluent = conc[2] + conc[3] * ss_eff * 1e-6 / ngr->ro_SS_sewage;

  r = 1.0 - ( c_effluent / c_influent );

  *f_dissolved_arg = conc[2] / c_effluent;
  *f_sorbed_arg = 1.0 - *f_dissolved_arg;

  return r;
}

/*
 * ---------------------------
 * 'generic' removal module 
 * --------------------------- 
*/


FloatType PrimRemoval ( FloatPtr wwtp_par_arg,
                        FloatType q_in_arg,
                        FloatType phi_in_arg,
                        FloatType dwf_arg,
                        FloatType* f_dissolved_arg,
                        FloatType* f_sorbed_arg )

{
  PrimParPtr p = (PrimParPtr) wwtp_par_arg;
  NonGeoRef_ParPtr ngr = NonGeoRefParams;
  FloatType r;
  FloatType ss_eff, R_SS_prim;
  FloatType f_tr1;
  FloatType c_in;

  switch ( CurrentWWTPComplMode )
  {
    case 1:

      r = ChemicalParams->R_primary;
      break;


    case 2:
    case 3:

      /* calculate inflow concentration */

      c_in = phi_in_arg / q_in_arg;

      /* fraction actually entering the WWTP */

      if ( p->capacity_prim != -1 )
        f_tr1 = FractionTreated ( p->capacity_prim, 
                                  dwf_arg,
                                  q_in_arg );
      else
        /* Intevation, 27-01-2003: Set treated fraction to 100 % by default
         * This fixes the Activated Sludge Mode 2 bug (no elimination was
         * calculated.)
         */
        f_tr1 = 1;	

      fTreatedPrimary = f_tr1;    /* remember this for use in AS model */

      /* add SS mass flux of this treatment plant to the total SS mass flux */

      R_SS_prim = SettlerEfficiency ( p->R_SS_prim_max,
                                      ngr->a_prim,
                                      q_in_arg * f_tr1,
                                      dwf_arg,
                                      ngr->SS_sewage );

      ss_eff = ( 1 - R_SS_prim ) * ngr->SS_sewage;

      PrimaryEffluentSS = ss_eff;    /* remember this for use in AS or TF model */

      /* calculate removal */

      r = PrimarySimpleTreat ( wwtp_par_arg,
                               q_in_arg * f_tr1,
                               phi_in_arg * f_tr1,
                               dwf_arg,
                               f_dissolved_arg,
                               f_sorbed_arg,
                               R_SS_prim );

      PurePrimaryRemoval = r;   /* keep this value in memory for the AS or TF model */

      r *= f_tr1;

      break;

    default:
      r = 0;
      break;
  }

  return r;
}



/*
 * **********************************************************************
 *
 * activated sludge
 *
 * **********************************************************************
*/

/*
 * ---------------------------
 * Modified SimpleTreat 3.0
 * 6-box model
 * --------------------------- 
*/

FloatType GreaterSimpleTreat ( FloatPtr wwtp_par_arg,
                               FloatType q_in_arg,
                               FloatType phi_in_arg,
                               FloatType dwf_arg,
                               FloatType f_dissolved_arg,
                               FloatType f_sorbed_arg,
                               FloatType R_SS_sec_arg )
{
  ASParPtr p = (ASParPtr) wwtp_par_arg;
  NonGeoRef_ParPtr ngr = NonGeoRefParams;
  Chemical_ParPtr chem = ChemicalParams;

  int i,j;

  FloatType r;
  FloatType yield;
  FloatType r_as_bod;
  FloatType prim_corr_BOD;
  FloatType prim_corr_N;
  FloatType person_equivalent;
  FloatType phi_bod;
  FloatType phi_n;
  FloatType v_act_sludge;
  FloatType a_act_sludge;
  FloatType a_sec;
  FloatType v_sec;
  FloatType hrt_act_sludge;
  FloatType phi_SS_surplus;
  FloatType ss_eff;
  FloatType srt_act_sludge = 0;
  FloatType oxreq;
  FloatType k_v, k_str, gpc;
  FloatType k_WWTP_deg;
  FloatType a_redox, a_temp;
  FloatType k_monod, k_max, c_st_st, srt_crit;
  FloatType c_influent, c_effluent;
  FloatType diff;
  FloatType sum;
  FloatType v[7];
  FloatType k[7];
  FloatType adv[7][7];
  FloatType xch[7][7];
  FloatType conc[7];
  FloatType phi[7];
  FloatType _coef[6][6];
  FloatPtr  _c = &(conc[1]);
  FloatPtr  _phi = &(phi[1]);

  /* initialize */

  for ( i=0 ; i<7 ; i++ )
  {
    v[i] = 0;
    k[i] = 0;
    conc[i] = 0;
    phi[i] = 0;

    for ( j=0 ; j<7 ; j++ )
    {
      adv[i][j] = 0;
      xch[i][j] = 0;
    }
  }

  for ( i=0 ; i<6 ; i++ )
    for ( j=0 ; j<6 ; _coef[i][j++] = 0 );
  
  /*
   * --------------------------------------------- *
   * estimation of design and operation parameters *
   * --------------------------------------------- *
  */

  /* effluent solids */

  ss_eff = p->SS_ML * ( 1.0 - R_SS_sec_arg );

  /* influent concentration */

  if ( q_in_arg > 0 )
    c_influent = phi_in_arg / q_in_arg ;
  else
    return 0;

  /*
   * AS TANK
   *
  */

  /* efficiency and yield, after Mikkelsen */

  r_as_bod = 0.818 - 0.0422 * log ( p->B_X );

  if ( p->Y == -1 )
    yield = 0.947 + 0.0739 * log ( p->B_X );    /* NB this doesn't feel right ! */
  else
    yield = p->Y;                               /* user supplied yield */

  /* water quality parameters influxes */

  prim_corr_BOD = ( p->h_prim != 0 ? 1.0 - ngr->R_prim_BOD : 1 );
  prim_corr_N   = ( p->h_prim != 0 ? 1.0 - ngr->R_prim_N   : 1 );

  if ( p->nr_PE == -1 )
    person_equivalent = dwf_arg / WaterUse;
  else
    person_equivalent = p->nr_PE;

  phi_bod = ngr->PHI_cap_BOD * 1.1574e-5 * person_equivalent * prim_corr_BOD; /* g/s */
  phi_n   = ngr->PHI_cap_N   * 1.1574e-5 * person_equivalent * prim_corr_N;   /* g/s */

  /* AS tank dimensions */

  v_act_sludge = phi_bod / ( 1.1574e-5 * p->B_X * p->SS_ML );    /* m3 */
  a_act_sludge = v_act_sludge / p->h_act_sludge;                 /* m2 */
  hrt_act_sludge = v_act_sludge / q_in_arg ;                     /* seconds */
  hrt_act_sludge *= 2.777777777778e-4;                           /* hours */

  /* oxygen requirement */  

  oxreq =   /* oxydation of organics */                          /* g/m3 */
        (   phi_bod * ( 1.0 - 1.33 * yield ) 

            /* endogenous respiration */
          + v_act_sludge * p->SS_ML * ngr->r_end / 3600     
                                                                 /* r_end is in g/g.h 
                                                                    an dis converted to g/g.sec */

            /* nitrification / denitrification */
          + ( phi_n - yield*phi_bod*0.05 ) 
            * ( 4.33*p->eps_N - 2.86*p->eps_N*p->eps_DN )
        )
          / q_in_arg;

  /* surplus sludge production (= waste sludge flow) */

  phi_SS_surplus = phi_bod * r_as_bod * yield - q_in_arg * ss_eff;   /* g/s */
  if ( phi_SS_surplus < 0 ) phi_SS_surplus = 0;

  /* SRT is calculated only if the Monod model is used ! */

  if ( (int)chem->use_Monod )
  {
    srt_act_sludge =  v_act_sludge * p->SS_ML                        /* seconds */
                      /
                      ( phi_SS_surplus + ss_eff * q_in_arg );

    srt_act_sludge *= 2.777777777778e-4;                             /* hours */
  }

  /*
   * SEC. SETTLER
   *
  */

  a_sec = dwf_arg / p->v_overflow_sec;                              /* m2 */
  v_sec = a_sec * p->h_sec;                                         /* m3 */


  /*
   * -------------------- *
   * influent calculation *
   * -------------------- *
  */

  phi[2] = f_dissolved_arg * phi_in_arg;
  phi[3] = f_sorbed_arg * phi_in_arg;

  /*
   * -------------------- *
   * advective transport  *
   * -------------------- *
  */

  /* air advection */

  adv[0][1] = ngr->h_air * ngr->wind_speed * sqrt( a_act_sludge + a_sec );
  adv[1][0] = adv[0][1];

  /* water volume flow rates */

  adv[0][2] = q_in_arg;
  adv[4][0] = adv[0][2];
  adv[4][2] = p->recycle * q_in_arg;
  adv[2][4] = (1 + p->recycle) * q_in_arg;

  adv[2][1] = 0;    /* aerosol formation not implemented ! */

  /* solids volume flow rate */

  adv[0][3] = ngr->SS_sewage * 1e-6 * ( 1.0 / ngr->ro_SS_sewage ) * q_in_arg;
  adv[3][5] = p->SS_ML * 1e-6 * ( 1.0 / ngr->ro_SS_ML ) * q_in_arg * ( 1 + p->recycle );
  adv[5][6] = R_SS_sec_arg * adv[3][5];
  adv[5][0] = ( 1.0 - R_SS_sec_arg ) * adv[3][5];
  adv[6][0] = phi_SS_surplus * 1e-6 * ( 1.0 / ngr->ro_SS_ML );
  adv[6][3] = adv[5][6] - adv[6][0];


  /*
   * -------------------- *
   * diffusive exchange   *
   * -------------------- *
  */

  /* box volumes */
  /* ----------- */

  v[1] = (  a_act_sludge + a_sec ) * ngr->h_air;
  v[2] = v_act_sludge;
  v[3] = v_act_sludge * p->SS_ML * 1e-6 * ( 1.0 / ngr->ro_SS_ML );
  v[4] = v_sec;
  v[5] = v_sec * ss_eff * 1e-6 * ( 1.0 / ngr->ro_SS_ML );
  /* v[6] need not be known ! */

  /* media volume flow rates */
  /* ----------------------- */

  /**** sludge / water */

  /* aeration tank */

  diff = ngr->k_sorb_ML / ( 1.0/(v[2]*FugacityWater) + 1.0/(v[3]*FugacityML) );
  xch[2][3] = diff / FugacityWater;
  xch[3][2] = diff / FugacityML;

  /* sec. settler */

  diff = ngr->k_sorb_sec / ( 1.0/(v[4]*FugacityWater) + 1.0/(v[5]*FugacityML) );
  xch[4][5] = diff / FugacityWater;
  xch[5][4] = diff / FugacityML;

  /**** air / water */

  /* sec. settler */

  diff = a_sec / ( 1.0/(ngr->K_air*FugacityAir) + 1.0/(ngr->K_water*FugacityWater) );
  xch[1][4] = diff / FugacityAir;
  xch[4][1] = diff / FugacityWater;

  /* aeration tank */

  /*--- volatilization */

  k_v =   ( 1.0 / ngr->h_air + CP.K_H / p->h_act_sludge )
        / ( 1.0 / ngr->K_air + CP.K_H / ngr->K_water );

  /*--- stripping */

  if ( p->aer_type == 1 )
  {
    /* SURFACE aeration */

    if ( CP.H >= 250 )
      gpc = 1;
    else
      gpc = ngr->kGa_kLa * CP.K_H / ( ngr->kGa_kLa * CP.K_H + 1 );

    k_str = ngr->PSI * gpc * oxreq 
                             / 
                             ( p->f_aerobic * hrt_act_sludge * ( DOSaturationML - p->DO ) );
  }
  else
  {
    /* BUBBLE aeration */

    k_str = 0.89e-3 * oxreq / ( p->f_aerobic * v_act_sludge ) * pow ( CP.H, 1.04 );
  }

  /*--- calculation of xch */

  diff =    k_str 
                / (   1.0 / ( v[1] * a_act_sludge/(a_act_sludge+a_sec) * FugacityAir ) 
                    + 1.0 / ( p->f_aerobic * v[2] * FugacityWater )
                  )
         +  k_v 
                / (   1.0 / ( v[1] * a_act_sludge/(a_act_sludge+a_sec) * FugacityAir ) 
                    + 1.0 / ( v[2] * FugacityWater )
                  );

  xch[1][2] = diff / FugacityAir;
  xch[2][1] = diff / FugacityWater;


  /*
   * ----------------------- *
   * degradation in AS tank  *
   * ----------------------- *
  */

  /* corrections for t and redox */
  /* --------------------------- */

  a_redox = p->f_aerobic + p->f_anoxic*chem->a_anoxic + p->f_anaerobic*chem->a_anaerobic;
  a_temp = pow ( chem->Q10, ( ngr->temp_sewage - 20.0 ) / 10.0 );

  if ( !((int)chem->use_Monod) )
  {

    /* double first order */
    /* ------------------ */

    k_WWTP_deg = (   chem->k_std_biodeg * p->SS_ML * chem->a_AS * a_redox  
                   + chem->k_WWTP_nonbio  ) * a_temp;

    /* rates per hour */

    k[2] = k_WWTP_deg;
    k[3] = k_WWTP_deg * chem->a_sorbed;
  }
  else
  {
   
    /* Monod kinetics */
    /* -------------- */

    k_max = chem->k_max * a_temp * a_redox;

    srt_crit = 1                        /* hours */
               /
               ( chem->Y * k_max * c_influent / ( chem->K_s + c_influent )  -  chem->b  );

    if ( srt_act_sludge < srt_crit )
    {
      k_monod = 0;
    }
    else
    {
      c_st_st = ( 1.0 + chem->b * srt_act_sludge ) * chem->K_s
                /
                ( srt_act_sludge * ( chem->Y * ( k_max - chem->b ) ) - 1.0 );

      k_monod = log ( c_influent / c_st_st ) / hrt_act_sludge;  /* per hour */
    }

    k[2] = k[3] = k_monod;  /* note that Monod degradation is assumed to take place
                               in both phases simultaneously ! */
  }

  /* convert rates to per second */

  k[2] *= 2.77777777e-4;
  k[3] *= 2.77777777e-4;


  /*
   * --------------------------------- *
   * fill in and solve the matrices    *
   * --------------------------------- *
  */

  /* note: _coef[0][0] corresponds with adv[1][1] etc. */

  for ( i=1 ; i<7 ; i++ )
    for ( j=1 ; j<7 ; j++ )
      if ( i != j)
        _coef[i-1][j-1] = - ( adv[j][i] + xch[j][i] );

  for ( i=1 ; i<7 ; i++ )
  {
    for ( sum=0, j=1 ; j<7 ; j++ )
      sum += ( adv[i][j] + xch[i][j] );

    _coef[i-1][i-1] = k[i]*v[i] + sum + adv[i][0];
  }


  /* solver: _coef * _c = _phi */
  /* linear system A * X = B  ->  arguments = A, B, X and n (with A - n x n matrix) */

  SolveLinearSystem ( (FloatType*)_coef, (FloatType*)_phi, (FloatType*)_c, 6);

  c_effluent = conc[4] + conc[5] * ss_eff * 1e-6 / ngr->ro_SS_ML;

  r = 1.0 - ( c_effluent / c_influent );

  return r;
}

/*
 * ---------------------------
 * 'generic' removal module 
 * --------------------------- 
*/


FloatType ASRemoval ( FloatPtr  wwtp_par_arg,
                      FloatType q_in_arg,
                      FloatType phi_in_arg,
                      FloatType dwf_arg )
{
  ASParPtr p = (ASParPtr) wwtp_par_arg;
  NonGeoRef_ParPtr ngr = NonGeoRefParams;
  Chemical_ParPtr chem = ChemicalParams;

  FloatType r1, r2, r;
  FloatType ss_eff, R_SS_sec;
  FloatType f_dissolved;
  FloatType f_sorbed;
  FloatType f_tr, f_tr2;

  switch ( CurrentWWTPComplMode )
  {
    case 1:

      /* is there a primary settler too ?
         if so, first do the primary settler calculations before proceeding with the AS */

      if ( (int) p->primary_present )
        r1 = PrimRemoval ( wwtp_par_arg, 
                           q_in_arg,
                           phi_in_arg, 
                           dwf_arg,
                            &f_dissolved,      /* f_dissolved and f_sorbed are to be calculated */
                            &f_sorbed );       /* by the prim. settler model ! */
      else
        r1 = 0;

      /* fraction actually entering the AS WWTP */

      f_tr2 = 1;
      f_tr = 1;

      /* calculate AS removal */

      r2 = chem->R_activated_sludge;

      /* calculate total removal of primary + secondary treatment */

      r = r1 + r2 - r1 * r2;     /* no bypasses in mode 1 
                                    calculatrion method explained for mode 2,3 */


      break;

    case 2:
    case 3:

      /* is there a primary settler too ?
         if so, first do the primary settler calculations before proceeding with the AS */

      if ( (int) p->primary_present )
      {
        if ( p->capacity_prim == -1 )
          p->capacity_prim = p->capacity_sec;

        PrimRemoval ( wwtp_par_arg, 
                      q_in_arg,
                      phi_in_arg, 
                      dwf_arg,
                      &f_dissolved,      /* f_dissolved and f_sorbed are to be calculated */
                      &f_sorbed );       /* by the prim. settler model ! */

        /* value of PrimRemoval is the TOTAL REMOVAL EFFICIENCY of the primary settler,
           taking into account lower removal due to settler bypassing;
           this is not useful for us at the moment as we want to recalculate this later on.
           BUT: we have kept the following in memory: */

        r1 = PurePrimaryRemoval;

      }
      else
      {
        /* create a hypothetical prim. settler with no removal */
        r1 = 0;

        /* 100% of flow goes through the no-removal hypothetical primary settler */
        fTreatedPrimary = 1;          

        /* the hypothetical prim. settler effluent = raw sewage ! */
        PrimaryEffluentSS = ngr->SS_sewage;

        /* sorbed / dissolved partitioning based on total equilibrium in sewage */
        f_dissolved = 1.0 / ( 1.0 + 1e-6 * ngr->SS_sewage * CP.Kd_sewage );
        f_sorbed    = 1.0 - f_dissolved;
      }


      /* fraction actually entering the AS WWTP */


      /* f_tr2 is the fraction of primary effluent going to secondary treatment */
      f_tr2 = FractionTreated ( p->capacity_sec, 
                                dwf_arg,
                                q_in_arg * fTreatedPrimary );

      /* f_tr is the fraction of total plant influent entering secondary treatment */
      f_tr = fTreatedPrimary * f_tr2;

      /* add SS mass flux of this treatment plant (prim + AS) to the total SS mass flux */

      R_SS_sec = SettlerEfficiency ( p->R_SS_sec_max,
                                     ngr->a_sec,
                                     q_in_arg * f_tr,
                                     dwf_arg,
                                     p->SS_ML );

      ss_eff = ( 1.0 - R_SS_sec ) * p->SS_ML;

      /* calculate AS removal */


      r2 = GreaterSimpleTreat ( wwtp_par_arg,
                                f_tr * q_in_arg,
                                (1 - r1) * f_tr * phi_in_arg,
                                dwf_arg,
                                f_dissolved,
                                f_sorbed,
                                R_SS_sec );


      /* calculate total removal of primary + secondary treatment */

      /* r = 1 - ( phi_out / phi_in )

         phi_out =    (1 - f1) * phi_in                          primary bypass
                    + [f1 * (1 - r1)] * (1 - f2) * phi_in        secondary bypass
                    + [f1 * (1 - r1)] * f2 * (1 - r2) * phi_in   secondary effluent
     
         hence:
         r = 1 - { (1 - f1) + [f1 * (1 - r1)] * (1 - f2) + [f1 * (1 - r1)] * f2 * (1 - r2) }
        
           = 1 - { 1 - f1 + (f1 - f1*r1) * (1 - f2) + (f1 - f1*r1) * (f2 - f2*r2) }

           = 1 - { 1 - f1 + f1 - f1*f2 - f1*r1 + f1*f2*r1 + f1*f2 - f1*f2*r2 -f1*f2*r1 + f1*f2*r1*r2 }

           = 1 - 1 + f1 - f1 + f1*f2 + f1*r1 - f1*f2*r1 - f1*f2 + f1*f2*r2 +f1*f2*r1 - f1*f2*r1*r2
      
           = f1*r1 + f1*f2*r2 - f1*f2*r1*r2

           = f1*r1 + ftotal*r2 - ftotal*r1*r2

      */

      r = fTreatedPrimary*r1 + f_tr*r2 - f_tr*r1*r2;

      break;

    default:
      r = 0;

      break;
  }

  return r;
}


/*
 * **********************************************************************
 *
 * trickling filter 
 *
 * **********************************************************************
*/


/*
 * ---------------------------
 * 'generic' removal module 
 * --------------------------- 
*/

FloatType TFRemoval ( FloatPtr  wwtp_par_arg,
                      FloatType q_in_arg,
                      FloatType phi_in_arg,
                      FloatType dwf_arg )
{
  TFParPtr p = (TFParPtr) wwtp_par_arg;
  Chemical_ParPtr chem = ChemicalParams;

  FloatType r1, r2, r;
  FloatType f_tr, f_tr2;

  switch ( CurrentWWTPComplMode )
  {
    case 1:

    /* ALWAYS mode 1 model, even if mode 2 or 3 are selected */
    case 2:
    case 3:

      /* is there a primary settler too ?
         if so, first do the primary settler calculations before proceeding with the TF */

      if ( (int) p->primary_present )
        r1 = chem->R_primary;             /* we don't call PrimRemoval because it may be not mode 1 ! */
      else
        r1 = 0;

      /* fraction actually entering the TF WWTP */

      f_tr2 = 1;
      f_tr = 1;

      /* calculate TF removal */

      r2 = chem->R_trickling_filter;

      /* calculate total removal of primary + secondary treatment */

      r = r1 + r2 - r1 * r2;     /* no bypasses in mode 1 
                                    calculatrion method explained for mode 2,3 */


      break;

    default:
      r = 0;
      break;
  }

  return r;
}


/*
 * ----------------------------------------------------------------------
 * SUMMARY of WWTP calculations
 * ----------------------------------------------------------------------
*/


FloatType WWTPRemoval ( FloatPtr  process_x_in_arg,
                        FloatPtr  process_par_arg,
                        FloatPtr  general_par_arg,
                        FloatType leak_arg,
                        int       submodel_arg )
{
  Treatment_ParPtr p = (Treatment_ParPtr) process_par_arg;
  WWTP_ParPtr q;
  FloatType r=0;
  FloatType WWTP_frac=0;
  FloatType dummy1, dummy2;
  int i;
  int no_exception_found = 1;

  switch (submodel_arg)
  {
    case TREATED:

      /* do we have site-specific data ? */
    
      if ( UseExceptions && NrWWTPExceptions )
      {
        for ( i=0 ; i<NrWWTPExceptions ; ++i )
          if ( WWTPExceptions[ i*2 ] == ThisSegmentID )
          {
            r = WWTPExceptions[ i*2 + 1 ];
            no_exception_found = 0;
          }
      }

      if ( no_exception_found )
      {
        for ( i=0 ; i<NR_WWTP_CLASSES ; ++i )
        {
          /* WWTP_frac is the fraction going to each WWTP in the Geogr. Unit
             note that sum(WWTP_frac[i]) is <= 1, it is <1 if <100% of the GU's
             waste water gets treatment */
  
          WWTP_frac = p->WWTP[i][1];
  
          /* q is the pointer to the WWTP's data set */
          q = (WWTP_ParPtr) &(WWTPData[ ThisSegmentMap->wwtp[i] * NR_WWTP_PARAMS ]);
  
  
          /**************************/
          /* which mode do we use ? */
          /**************************/
        
          if ( ImposeComplexityModeWWTP || q->mode == -1 )
          {
            if ( ComplexityModeRiver != -1 )
              CurrentWWTPComplMode = ComplexityModeWWTP;
            else
              CurrentWWTPComplMode = 1; /* if nothing is specified, take mode 1 as default */
          }
          else
            CurrentWWTPComplMode = q->mode;
  
  
          switch ( (int) q->type)
          {
            
            /* note: WWTP_DryWeatherFlow was calculated in MODEL.C, function Model_Sewer */
  
            case PRIMARY:
  
              /* call primary module */
  
              r += WWTP_frac 
                   * PrimRemoval ( (FloatPtr)q, 
                                   WWTP_frac * process_x_in_arg[ _Q_*OneStateVarSize ],
                                   WWTP_frac * process_x_in_arg[ _PHI_*OneStateVarSize ],
                                   WWTP_frac * WWTP_DryWeatherFlow,
                                   &dummy1,
                                   &dummy2 );
  
              break;
  
            case ACTIVATED_SLUDGE:
  
              /* call AS module */
  
              r += WWTP_frac 
                   * ASRemoval ( (FloatPtr)q,
                                 WWTP_frac * process_x_in_arg[ _Q_*OneStateVarSize ],
                                 WWTP_frac * process_x_in_arg[ _PHI_*OneStateVarSize ],
                                 WWTP_frac * WWTP_DryWeatherFlow );
              break;
  
            case TRICKLING_FILTER:
  
              /* call TF module */
            
              r += WWTP_frac 
                   * TFRemoval ( (FloatPtr)q,
                                 WWTP_frac * process_x_in_arg[ _Q_*OneStateVarSize ],
                                 WWTP_frac * process_x_in_arg[ _PHI_*OneStateVarSize ],
                                 WWTP_frac * WWTP_DryWeatherFlow );
              break;
  
            default:
              /* r doesn't increase */
              break;
          }
        }
      }
      break;

    case NONE:
      r = 0;
      break;

    default: 
      r = 0;
      break;
  }

  return r;
}


