/*
 * GREAT-ER
 *
 * TREEWALK.C
 *
 * walking the river network
 *
 * author =  Geert Boeije                                       
 *
 * changes by Intevation, http://intevation.net
 * state =   16.02.2000
 *
 * Intevation: replaced IN and OUT by IN__ and OUT__
*/

#include "init.h"
#include "model.h"
#include "error.h"
#include "usriface.h"
#include "util.h"
#include "navigate.h"
#include "treewalk.h"

#ifdef GUI_MODE

#include "extgui.h"
int Counter = 10000;

#endif

char c[1000];

/*
 * --------------------------------------------------------------------
 *
 * a discrete scenario (ie Monte Carlo shot) has been generated
 *
 * now, all segments are simulated using the generated parameters shot
 * 
 * --------------------------------------------------------------------
*/

void TreeWalk ( FloatPtr  allsegments_iface_arg, 
                FloatPtr  allsegments_summary_arg,
                FloatPtr  allsegments_par_arg,
                FloatPtr  allsegments_rivclass_arg,
                int       current_segment_arg )
{

  FloatPtr thissegment_par;
  Map_Ptr thissegment_map;
  General_ParPtr gen_par;
  FloatPtr  thissegment_rivclass;

  #ifdef GUI_MODE

  CheckUserBreak ();
  if (ExitCode)
    return;

  #endif

  /* if we have already calculated this segment, we go back to the previous step */

  if ( SegmentDone[ current_segment_arg ] )
    return;

  /* let's go */

  thissegment_par = &allsegments_par_arg[ SegmentParOffset [current_segment_arg] ];
  gen_par = (General_ParPtr)thissegment_par;

  thissegment_map = &Map[ current_segment_arg ];

  /* walk up the tree and get the influent */

  switch ( (int)(gen_par->segment_type) )
  {
    case NOTHING_SPECIAL:

      /* first, calculate the upstream stretch */

      TreeWalk ( allsegments_iface_arg,
                 allsegments_summary_arg,
                 allsegments_par_arg,
                 allsegments_rivclass_arg,
                 thissegment_map->upstream_one );

      /* get the river stretch's input from the upstream stretch */

      GetInflowFromUpstream ( current_segment_arg, 
                              thissegment_map->upstream_one,
                              -1, 
                              allsegments_iface_arg,
                              allsegments_par_arg    );

      break;
    
    case END_OF_TREE:

      break;
      
    case CONFLUENCE:
    
      /* first, calculate the upstream stretches */
      
      TreeWalk ( allsegments_iface_arg,
                 allsegments_summary_arg,
                 allsegments_par_arg,
                 allsegments_rivclass_arg,
                 thissegment_map->upstream_one );

      TreeWalk ( allsegments_iface_arg,
                 allsegments_summary_arg,
                 allsegments_par_arg,
                 allsegments_rivclass_arg,
                 thissegment_map->upstream_two );

      /* get the river stretch's input from the two upstream stretches */
      
      GetInflowFromUpstream ( current_segment_arg, 
                              thissegment_map->upstream_one,
                              thissegment_map->upstream_two,
                              allsegments_iface_arg,
                              allsegments_par_arg );

      break;

    case THE_SEA:
    
      /* just call the upstream stretches */
      
      TreeWalk ( allsegments_iface_arg,
                 allsegments_summary_arg,
                 allsegments_par_arg,
                 allsegments_rivclass_arg,
                 thissegment_map->upstream_one );

      TreeWalk ( allsegments_iface_arg,
                 allsegments_summary_arg,
                 allsegments_par_arg,
                 allsegments_rivclass_arg,
                 thissegment_map->upstream_two );

      break;

    case BIFURCATION:

      /* a bifurcation looks like this:

               upstream
              ____|____
             |         |
            bif_1     bif_2

         here, bif_1 and bif_2 are called bifurcation stretches.

         parameters are: upstream_one = the upstream stretch
                         upstream_two = the 'neighboring' stretch

         hence, for bif_1: upstream_one = upstream
                           upstream_two = bif_2
      */

      /* first go for the upstream stretch */

      TreeWalk ( allsegments_iface_arg,
                 allsegments_summary_arg,
                 allsegments_par_arg,
                 allsegments_rivclass_arg,
                 thissegment_map->upstream_one );

      /* now we get the inflow from upstream, taking into the account the bifurcation */

      GetBifurcationInflowFromUpstream ( current_segment_arg,
                                         thissegment_map->upstream_one,
                                         thissegment_map->upstream_two,
                                         allsegments_iface_arg,
                                         allsegments_par_arg );
      break;

    default: break;
  }

  /* model calc only if real stretch */

  if ( (int)(gen_par->segment_type) != THE_SEA )
  {
    /* find the rivclass data */

    thissegment_rivclass
      = &(allsegments_rivclass_arg [ thissegment_map->rivclass * NR_RIVCLASS_PARAMS ]);

    /* call the model for the current segment */

    CleanUpArrays ();      
      
    SegmentControlSection
       ( current_segment_arg,
         thissegment_par,
         thissegment_rivclass,
         &allsegments_iface_arg
                   [(NR_STATEVARS*NR_IFACE_TERMS)*current_segment_arg],
         &allsegments_summary_arg
                   [(NR_SUMMARY_VARS*NR_SUMMARY_TERMS)*current_segment_arg] );
  }

  /* remember we've done this segment */

  SegmentDone[ current_segment_arg ] = 1;
  
  #ifdef GUI_MODE

  /*
  if ( ++Counter > 500 )
  {
    PercentDone2 ();
    Counter = 0;
  }
  */

  CheckUserBreak ();
  if (ExitCode)
    return;

  #endif
}


/*
 * get the interface OUT values of the upstream segment(s)
 * and store the sum of all upstream values into the current
 * interface IN variable
*/

/* normal case: one or two up, one down */ 

void GetInflowFromUpstream ( int     to_nr_arg,
                             int     from_segment_one_arg,
                             int     from_segment_two_arg,
                             FloatPtr allsegments_iface_arg,
                             FloatPtr allsegments_par_arg    )
{
  FloatPtr from_iface;
  FloatPtr to_iface;
  int from_nr[ 2 ] = { -1, -1 };
  int i, j;

  from_nr[ 0 ] = from_segment_one_arg;
  from_nr[ 1 ] = from_segment_two_arg;

  to_iface = &allsegments_iface_arg[ (NR_STATEVARS * NR_IFACE_TERMS) * to_nr_arg ];

  for ( j=0 ; j<2 ; ++j )
    if ( from_nr[j] != -1 )
    {
      from_iface 
      = &allsegments_iface_arg[ (NR_STATEVARS * NR_IFACE_TERMS) * from_nr[j] ];
    
      for ( i=0 ; i<NR_STATEVARS ; ++i )
        to_iface[ i*NR_IFACE_TERMS + IN__ ] += from_iface[ i*NR_IFACE_TERMS + OUT__ ];
    }

  return;
}

/* bifurcation case: one up, two down */

void GetBifurcationInflowFromUpstream ( int     to_nr_arg,
                                        int     upstream_arg,
                                        int     neighbor_arg,
                                        FloatPtr allsegments_iface_arg,
                                        FloatPtr allsegments_par_arg    )
{
  FloatPtr from_iface;
  FloatPtr to_iface;
  River_ParPtr here;
  River_ParPtr neighbor;
  FloatType flow_fract;
  int i;

  /* get the flow information about this segment and its neighbour,
     and determine the flow fraction going to this segment             */ 

  here     = (River_ParPtr)(&allsegments_par_arg
                             [ SegmentParOffset [to_nr_arg]    
                             + ParametersArrayOffset[RIVER] ]);

  neighbor = (River_ParPtr)(&allsegments_par_arg
                             [ SegmentParOffset [neighbor_arg] 
                             + ParametersArrayOffset[RIVER] ]);

  flow_fract = here->Q / ( here->Q + neighbor->Q );

  /* copy the determined fraction to this segment */

  to_iface =   &allsegments_iface_arg[ (NR_STATEVARS * NR_IFACE_TERMS) * to_nr_arg ];
  from_iface = &allsegments_iface_arg[ (NR_STATEVARS * NR_IFACE_TERMS) * upstream_arg ];
    
  for ( i=0 ; i<NR_STATEVARS ; ++i )
    to_iface[ i*NR_IFACE_TERMS + IN__ ] 
      = from_iface[ i*NR_IFACE_TERMS + OUT__ ] * flow_fract;

  return;
}


/*
 * --------------------------------------------------------------------
 *
 * initialize the tree walking Map array
 *
 * --------------------------------------------------------------------
*/

int InitializeTree ( void )
{
  int i, seg, wwtp_code, temp;
  Map_Ptr this_map;
  General_ParPtr gen;
  FloatType err[2];

  for ( seg=0 ; seg < NrSegments ; ++seg )
  {
    this_map = &(Map[ seg ]);
    gen = (General_ParPtr)(&Distri_RiverParams[ seg*TotRiverDistriPar ]);

    /* river class */

    if ( (this_map->rivclass = RivClassNumber ( gen->rivclass ) ) == -1 )
    {
      err[0] = gen->segment_ID;
      err[1] = gen->rivclass;
      GenerateErrorMessage (506, err);
      return 0;
    }

    /* 1st upstream ref */

    if ( (int)(gen->segment_type) != END_OF_TREE )
      if ( (this_map->upstream_one = SegmentNumber ( seg, gen->upstream_one, _UP_ )) 
           == -1 )
      {
        err[0] = gen->segment_ID;
        err[1] = gen->upstream_one;

        GenerateErrorMessage (501, err );
        return 0;
      }
      else;
    else
      this_map->upstream_one = -1;

    /* 2nd upstream ref */


    if ( (int)(gen->segment_type) == BIFURCATION || 
         (int)(gen->segment_type) == CONFLUENCE )
      if ( (this_map->upstream_two = SegmentNumber ( seg, gen->upstream_two, _UP_ )) 
           == -1 )
      {
        err[0] = gen->segment_ID;
        err[1] = gen->upstream_two;

        GenerateErrorMessage ( (int)(gen->segment_type) == BIFURCATION ? 
                                502 : 501, err );
        return 0;
      }
      else;

    else
      this_map->upstream_two = -1;


    /* discharge ref
       we need: - geo-referenced discharge data
                - emission data                     
                - wwtp location in array        */

    if ( (int)(gen->discharge) )
    {
      /* geo-ref discharge (waste water pathway) data */  

      if ( (this_map->discharge = DischargeNumber ( seg, gen->segment_ID )) == -1 )
      {
        GenerateErrorMessage (503, &(gen->segment_ID) );
        return 0;
      }

      /* emission data */

      temp = this_map->discharge * TotDischargeDistriPar + Emission_ID_Location;
      temp = (int) Distri_DischargeParams[ temp ];

      if ( (this_map->emission = EmissionNumber ( temp )) == -1 )
      {
        GenerateErrorMessage (504, &(gen->segment_ID) );
        return 0;
      }

      /* wwtp */

      temp = this_map->discharge * TotDischargeDistriPar + WWTP_ID_Location;

      for ( i=0 ; i<NR_WWTP_CLASSES ; ++i )
      {
        wwtp_code = (int) Distri_DischargeParams[ temp + 2*i ];

        if ( wwtp_code == -1 )
          this_map->wwtp[i] = -1;
        else
          if ( (this_map->wwtp[i] = WWTPNumber ( wwtp_code )) == -1 )
          {
            err[0] = gen->segment_ID;
            err[1] = (FloatType)wwtp_code;
            GenerateErrorMessage (507, err );
            return 0;
          }
      }
    }
    else
    {
      this_map->discharge = -1;
      this_map->emission = -1;
      for ( i=0 ; i<NR_WWTP_CLASSES ; this_map->wwtp[i++] = -1 );
    }
  }

  return 1;
}



